DataTables example

DataTables individual column filtering example (using select menus)

# Preamble

This example is almost identical to individual column example and provides the same functionality, but using <select> menus rather than input elements. The API plug-in function fnGetColumnData from Benedikt Forchhammer provides much of the logic processing required, and integration with a table is almost trivial.

One possible interaction chance would be to make use of fnGetColumnData's ability to get filtered data, so you could have the possible filtering values in the select menus to update to only those in the table, rather than all values.

# Live example

Rendering engine | Browser | Platform(s) | Engine version | CSS grade
--- | --- | --- | --- | ---
Trident | Internet Explorer 4.0 | Win 95+ | 4 | X
Trident | Internet Explorer 5.0 | Win 95+ | 5 | C
Trident | Internet Explorer 5.5 | Win 95+ | 5.5 | A
Trident | Internet Explorer 6 | Win 98+ | 6 | A
Trident | Internet Explorer 7 | Win XP SP2+ | 7 | A
Trident | AOL browser (AOL desktop) | Win XP | 6 | A
Gecko | Firefox 1.0 | Win 98+ / OSX.2+ | 1.7 | A
Gecko | Firefox 1.5 | Win 98+ / OSX.2+ | 1.8 | A
Gecko | Firefox 2.0 | Win 98+ / OSX.2+ | 1.8 | A
Gecko | Firefox 3.0 | Win 2k+ / OSX.3+ | 1.9 | A
Gecko | Camino 1.0 | OSX.2+ | 1.8 | A
Gecko | Camino 1.5 | OSX.3+ | 1.8 | A
Gecko | Netscape 7.2 | Win 95+ / Mac OS 8.6-9.2 | 1.7 | A
Gecko | Netscape Browser 8 | Win 98SE+ | 1.7 | A
Gecko | Netscape Navigator 9 | Win 98+ / OSX.2+ | 1.8 | A
Gecko | Mozilla 1.0 | Win 95+ / OSX.1+ | 1 | A
Gecko | Mozilla 1.1 | Win 95+ / OSX.1+ | 1.1 | A
Gecko | Mozilla 1.2 | Win 95+ / OSX.1+ | 1.2 | A
Gecko | Mozilla 1.3 | Win 95+ / OSX.1+ | 1.3 | A
Gecko | Mozilla 1.4 | Win 95+ / OSX.1+ | 1.4 | A
Gecko | Mozilla 1.5 | Win 95+ / OSX.1+ | 1.5 | A
Gecko | Mozilla 1.6 | Win 95+ / OSX.1+ | 1.6 | A
Gecko | Mozilla 1.7 | Win 98+ / OSX.1+ | 1.7 | A
Gecko | Mozilla 1.8 | Win 98+ / OSX.1+ | 1.8 | A
Gecko | Seamonkey 1.1 | Win 98+ / OSX.2+ | 1.8 | A
Gecko | Epiphany 2.20 | Gnome | 1.8 | A
Webkit | Safari 1.2 | OSX.3 | 125.5 | A
Webkit | Safari 1.3 | OSX.3 | 312.8 | A
Webkit | Safari 2.0 | OSX.4+ | 419.3 | A
Webkit | Safari 3.0 | OSX.4+ | 522.1 | A
Webkit | OmniWeb 5.5 | OSX.4+ | 420 | A
Webkit | iPod Touch / iPhone | iPod | 420.1 | A
Webkit | S60 | S60 | 413 | A
Presto | Opera 7.0 | Win 95+ / OSX.1+ | - | A
Presto | Opera 7.5 | Win 95+ / OSX.2+ | - | A
Presto | Opera 8.0 | Win 95+ / OSX.2+ | - | A
Presto | Opera 8.5 | Win 95+ / OSX.2+ | - | A
Presto | Opera 9.0 | Win 95+ / OSX.3+ | - | A
Presto | Opera 9.2 | Win 88+ / OSX.3+ | - | A
Presto | Opera 9.5 | Win 88+ / OSX.3+ | - | A
Presto | Opera for Wii | Wii | - | A
Presto | Nokia N800 | N800 | - | A
Presto | Nintendo DS browser | Nintendo DS | 8.5 | C/A1
KHTML | Konqureror 3.1 | KDE 3.1 | 3.1 | C
KHTML | Konqureror 3.3 | KDE 3.3 | 3.3 | A
KHTML | Konqureror 3.5 | KDE 3.5 | 3.5 | A
Tasman | Internet Explorer 4.5 | Mac OS 8-9 | - | X
Tasman | Internet Explorer 5.1 | Mac OS 7.6-9 | 1 | C
Tasman | Internet Explorer 5.2 | Mac OS 8-X | 1 | C
Misc | NetFront 3.1 | Embedded devices | - | C
Misc | NetFront 3.4 | Embedded devices | - | A
Misc | Dillo 0.8 | Embedded devices | - | X
Misc | Links | Text only | - | X
Misc | Lynx | Text only | - | X
Misc | IE Mobile | Windows Mobile 6 | - | C
Misc | PSP browser | PSP | - | C
Other browsers | All others | - | - | U

# Initialisation code

    (function($) {
    /*
     * Function: fnGetColumnData
     * Purpose:  Return an array of table values from a particular column.
     * Returns:  array string: 1d data array
     * Inputs:   object:oSettings - dataTable settings object. This is always the last argument past to the function
     *           int:iColumn - the id of the column to extract the data from
     *           bool:bUnique - optional - if set to false duplicated values are not filtered out
     *           bool:bFiltered - optional - if set to false all the table data is used (not only the filtered)
     *           bool:bIgnoreEmpty - optional - if set to false empty values are not filtered from the result array
     * Author:   Benedikt Forchhammer <b.forchhammer /AT\ mind2.de>
     */
    $.fn.dataTableExt.oApi.fnGetColumnData = function ( oSettings, iColumn, bUnique, bFiltered, bIgnoreEmpty ) {
    	// check that we have a column id
    	if ( typeof iColumn == "undefined" ) return new Array();

    	// by default we only want unique data
    	if ( typeof bUnique == "undefined" ) bUnique = true;

    	// by default we do want to only look at filtered data
    	if ( typeof bFiltered == "undefined" ) bFiltered = true;

    	// by default we do not want to include empty values
    	if ( typeof bIgnoreEmpty == "undefined" ) bIgnoreEmpty = true;

    	// list of rows which we're going to loop through
    	var aiRows;

    	// use only filtered rows
    	if (bFiltered == true) aiRows = oSettings.aiDisplay;
    	// use all rows
    	else aiRows = oSettings.aiDisplayMaster; // all row numbers

    	// set up data array
    	var asResultData = new Array();

    	for (var i=0,c=aiRows.length; i<c; i++) {
    		iRow = aiRows[i];
    		var aData = this.fnGetData(iRow);
    		var sValue = aData[iColumn];

    		// ignore empty values?
    		if (bIgnoreEmpty == true && sValue.length == 0) continue;

    		// ignore unique values?
    		else if (bUnique == true && jQuery.inArray(sValue, asResultData) > -1) continue;

    		// else push the value onto the result data array
    		else asResultData.push(sValue);
    	}

    	return asResultData;
    }}(jQuery));

    function fnCreateSelect( aData )
    {
    	var r='<select><option value=""></option>', i, iLen=aData.length;
    	for ( i=0 ; i<iLen ; i++ )
    	{
    		r += '<option value="'+aData[i]+'">'+aData[i]+'</option>';
    	}
    	return r+'</select>';
    }

    $(document).ready(function() {
    	/* Initialise the DataTable */
    	var oTable = $('#example').dataTable( {
    		"oLanguage": {
    			"sSearch": "Search all columns:"
    		}
    	} );

    	/* Add a select menu for each TH element in the table footer */
    	$("tfoot th").each( function ( i ) {
    		this.innerHTML = fnCreateSelect( oTable.fnGetColumnData(i) );
    		$('select', this).change( function () {
    			oTable.fnFilter( $(this).val(), i );
    		} );
    	} );
    } );

# Other examples

## Basic initialisation

- Zero configuration
- Feature enablement
- Sorting data
- Multi-column sorting
- Multiple tables
- Hidden columns
- Complex headers - grouping with colspan
- DOM positioning
- Flexible table width
- State saving
- Alternative pagination styles
- Scrolling:
  Horizontal / Vertical / Both / Themed / Infinite
- Change language information (internationalisation)
- ThemeRoller themes (Smoothness)

## Advanced initialisation

- Events:
  Live events / Pre-init / Post-init
- Column rendering
- Sorting without HTML tags
- Multiple table controls (sDom)
- Defining length menu options
- Complex headers and hidden columns
- Custom toolbar (element) around table
- Row highlighting with CSS
- Row grouping
- Row callback
- Footer callback
- Control sorting direction of columns
- Change language information from a file (internationalisation)
- Setting defaults
- State saving with localStorage
- Custom events

## API

- Dynamically add a new row
- Individual column filtering (using "input" elements)
- Individual column filtering (using "select" elements)
- Highlight rows and columns
- Show and hide details about a particular record
- User selectable rows (multiple rows)
- User selectable rows (single row) and delete rows
- Editable rows (with jEditable)
- Submit form with elements in table
- Index column (static number column)
- Show and hide columns dynamically
- API function use in initialisation object (callback)
- DataTables scrolling and tabs
- Regular expression filtering

## Data sources

- DOM
- Javascript array
- Ajax source
- Server side processing

## Server-side processing

- Obtain server-side data
- Add extra HTTP variables
- Use HTTP POST
- Automatic addition of IDs and classes to rows
- Reading table data from objects
- Show and hide details about a particular record
- User selectable rows (multiple rows)
- JSONP for a cross domain data source
- jEditable integration with DataTables
- Deferred loading of Ajax data
- Pipelining data (reduce Ajax calls for paging)

## Ajax data source

- Ajax sourced data (array of arrays)
- Ajax sourced data (array of objects)
- Deferred DOM creation for extra speed
- Empty data source columns
- Use a data source other than aaData (the default)
- Read column data from sub-arrays
- Read column data from deeply nested properties

## Plug-ins

- Add custom API functions
- Sorting and automatic type detection
- Sorting without automatic type detection
- Custom pagination controls
- Range filtering / custom filtering
- Live DOM sorting
- Automatic HTML type detection